package cn.com.qmth.examcloud.core.basic.api.controller;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.helpers.tree.EleTreeNode;
import cn.com.qmth.examcloud.commons.web.helpers.tree.TreeUtil;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.PrivilegeGroupBean;
import cn.com.qmth.examcloud.core.basic.dao.AppRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeGroupRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.App;
import cn.com.qmth.examcloud.core.basic.dao.entity.Privilege;
import cn.com.qmth.examcloud.core.basic.dao.entity.PrivilegeGroup;
import io.swagger.annotations.ApiOperation;

/**
 * 权限
 *
 * @author WANGWEI
 * @date 2018年6月12日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@RestController
@RequestMapping("${app.api.root}/privilege")
public class PrivilegeController extends ControllerSupport {

	@Autowired
	AppRepo appRepo;

	@Autowired
	PrivilegeGroupRepo privilegeGroupRepo;

	@Autowired
	PrivilegeRepo privilegeRepo;

	@Autowired
	OrgRepo orgRepo;

	@ApiOperation(value = "查询权限组")
	@GetMapping("getPrivilegeGroupList")
	public List<PrivilegeGroupBean> getPrivilegeGroupList() {
		Sort sort = new Sort(Sort.Direction.ASC, "id");
		List<PrivilegeGroup> list = privilegeGroupRepo.findAll(sort);

		List<PrivilegeGroupBean> ret = Lists.newArrayList();

		for (PrivilegeGroup cur : list) {
			PrivilegeGroupBean bean = new PrivilegeGroupBean();
			bean.setId(cur.getId());
			bean.setAppId(cur.getAppId());
			App app = appRepo.findOne(cur.getAppId());
			if (null != app) {
				bean.setAppName(app.getName());
			}
			bean.setCode(cur.getCode());
			bean.setName(cur.getName());

			ret.add(bean);
		}

		return ret;
	}

	@ApiOperation(value = "查询权限树")
	@GetMapping("getPrivilegeTree/{groupId}")
	public EleTreeNode getPrivilegeTree(@PathVariable Long groupId) {
		PrivilegeGroup group = privilegeGroupRepo.findOne(groupId);
		List<Privilege> privilegeList = privilegeRepo.findAllByGroupId(groupId);
		EleTreeNode rootNode = new EleTreeNode(-1L, group.getName());
		TreeUtil.convert2OneEleTreeNode(rootNode, privilegeList);

		return rootNode;
	}

	@ApiOperation(value = "增加权限")
	@PostMapping("addPrivilege")
	public Privilege addPrivilege(@RequestBody Privilege privilege) {

		privilege.setName(privilege.getName().trim());
		privilege.setCode(privilege.getCode().trim());
		privilege.setDescription(privilege.getDescription());

		PrivilegeGroup group = privilegeGroupRepo.findOne(privilege.getGroupId());
		if (null == group) {
			throw new StatusException("B-020001", "权限组不存在");
		}

		if (null != privilege.getParentId()) {
			Privilege parentPrivilege = privilegeRepo.findOne(privilege.getParentId());
			if (null == parentPrivilege) {
				throw new StatusException("B-020002", "父权限不存在");
			}
			if (!parentPrivilege.getGroupId().equals(privilege.getGroupId())) {
				throw new StatusException("B-020003", "权限组错误");
			}
		}

		Privilege ret = privilegeRepo.save(privilege);

		return ret;
	}

	@ApiOperation(value = "更新权限")
	@PutMapping("updatePrivilege")
	public Privilege updatePrivilege(@RequestBody Privilege privilege) {

		privilege.setName(privilege.getName().trim());
		privilege.setCode(privilege.getCode().trim());
		privilege.setDescription(privilege.getDescription());
		PrivilegeGroup group = privilegeGroupRepo.findOne(privilege.getGroupId());
		if (null == group) {
			throw new StatusException("B-020001", "权限组不存在");
		}

		if (null != privilege.getParentId()) {
			Privilege parentPrivilege = privilegeRepo.findOne(privilege.getParentId());
			if (null == parentPrivilege) {
				throw new StatusException("B-020002", "父权限不存在");
			}
			if (parentPrivilege.getGroupId().equals(privilege.getGroupId())) {
				throw new StatusException("B-020003", "权限组错误");
			}
		}

		Privilege ret = privilegeRepo.save(privilege);

		return ret;
	}

	@ApiOperation(value = "删除权限")
	@DeleteMapping("deletePrivilege/{id}")
	public void deletePrivilege(@PathVariable Long id) {
		Privilege privilege = privilegeRepo.findOne(id);
		if (null == privilege) {
			throw new StatusException("B-020004", "权限不存在");
		}

		List<Privilege> children = privilegeRepo.findAllByParentId(privilege.getId());
		if (CollectionUtils.isNotEmpty(children)) {
			throw new StatusException("B-020005", "存在子权限");
		}
		privilegeRepo.delete(id);
	}

	@ApiOperation(value = "查询权限")
	@GetMapping("getPrivilege/{id}")
	public Privilege getPrivilege(@PathVariable Long id) {
		Privilege privilege = privilegeRepo.findOne(id);
		if (null == privilege) {
			throw new StatusException("B-020004", "权限不存在");
		}
		return privilege;
	}

}
